function aiyagarical3b = aiyagarical3b(z)
%function that solves for equilibrium rs, wu, ws, and calibrates B, betas, betau, omega, nu, lambda, meany 
%in an Aiyagari model with complementarity and non-linear labor taxes. re is given by NAC
global nA A a m alpha eta omega rho nu k0 ds de lu ls rs re tauke tauks ks ke taul nY sigma Yu Ys asprime auprime ...
    lambdas lambdau lambdasa lambdaua gy x pis piu Ts Tu Amat Tlambdau Tlambdas tauc by lspol lambdal lupol lsinv luinv cspol cupol R


z; %just a potential check

%defining variables
rs = z(1);
r = rs;
w = [z(2), z(3)];
re = (1-tauks)*(rs - ds)/(1-tauke) + de;%the NAC
lambdal = z(4);
B = z(5);
betas = z(6);
omega = z(7);
nu = z(8);
betau = z(9);
meany = z(10);

%defining net returns to be used below to avoid extra computations
R = 1+(r-ds)*(1-tauks);
%defining after tax labor income to be used below
Ysmat = lambdal*(kron(Ys*w(2), ones(nA,1)).*lspol/meany).^(1-taul)*meany;
Yumat = lambdal*(kron(Yu*w(1), ones(nA,1)).*lupol/meany).^(1-taul)*meany;
Ysinv = lambdal*(kron(Ys*w(2), ones(nA,1)).*lsinv/meany).^(1-taul)*meany;
Yuinv = lambdal*(kron(Yu*w(1), ones(nA,1)).*luinv/meany).^(1-taul)*meany;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%loop on finding policies given fixed prices
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
differ = 1;
n = 0;
nmax = 10^6;

%tic
while (differ > 10^-9)&(n<nmax);%everything in matrix form, time savings 1/9
    %seems a high precision is needed here,at least -9

    as = (((betas*R*Ts*((Ysmat + R*Amat - asprime).^(-sigma))')').^(-1/sigma)+ Amat - Ysinv)/R;
    au = (((betau*R*Tu*((Yumat + R*Amat - auprime).^(-sigma))')').^(-1/sigma)+ Amat - Yuinv)/R;


    ainvs = as;
    ainvu = au;
    %Putting the new policy function back on the grid. Need an
    %approximation outside of the grid points for that.
    for k = 1:nA
       for m = 1:nY
           %anew(k,m) = interp1(a(:, m), A, A(k), 'linear', 'extrap'); %using linear interpolation in Matlab
           %anew(k,m) = interp1(a(:, m), A, A(k), 'cubic', 'extrap'); %using cubic interpolation in Matlab, the slowest
           %a = as;
           %asnew(k,m) = bfendo(A(k))*A; %using my own linear interpolation, the fastest!!!
           %a = au;
           %aunew(k,m) = bfendo(A(k))*A; %using my own linear interpolation, the fastest!!!
           
           asnew(k,m) = bfendo2(A(k), nA, m, as)*A; %put everything in the function, global variables costly
           aunew(k,m) = bfendo2(A(k), nA, m, au)*A; 
        end
    end
    asnew = max(asnew, min(A));
    asnew = min(asnew, max(A)); 
    aunew = max(aunew, min(A));
    aunew = min(aunew, max(A)); 
    differ = sum([sum(abs(asnew-asprime)), sum(abs(aunew-auprime))]);
    asprime = asnew;
    auprime = aunew;
    n = n + 1;
end

if n == nmax;
n
end
%n
%display 'Solving for policies takes'
%toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%here I am computing stat. distribution 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%display 'now computing the transition matrix'

diff = 1;
m=0;
a = Amat;

%tic
%this here only generates the transition matrix
    Tlambdas = zeros(nA*nY, nA*nY);
    Tlambdau = zeros(nA*nY, nA*nY);
    for i = 1:nA
        for m = 1:nY
            
             %this is the old version of this function, but this is not used many times
             loms = max(min(bfendo(asprime(i,m)),1),0);%to get probabilities in [0,1]
             lomu = max(min(bfendo(auprime(i,m)),1),0);
                                
             %mind the order here, don't compute the bfendo's unnecessarily

             for k = 1:nA
                for l = 1:nY
                    Tlambdas(nY*(i-1)+m,nY*(k-1)+l) = loms(k)*Ts(m,l);
                    Tlambdau(nY*(i-1)+m,nY*(k-1)+l) = lomu(k)*Tu(m,l);
                end
            end
            
            
            
        end
    end
%display 'Computing the transition matrix takes'
%toc

% Tlambdas
% Tlambdau
% bla


%tic;
%it is usually faster than the proceure without s, but seems not to work in some cases - need to verify eigen value = 1
options.disp = 0;
[vector value] = eigs(Tlambdas.',1,1,options);
lambdas = vector(:,1)/sum(vector(:,1))*pis;
[vector value] = eigs(Tlambdau.',1,1,options);
lambdau = vector(:,1)/sum(vector(:,1))*piu;
%display 'Computing the stationary distribution takes'
%toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Now we have the policies and stationary distributions, so we can compute
%steady state allocations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%constructing labor policies
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lambdasmat = (reshape(lambdas, nY, nA))';
lambdaumat = (reshape(lambdau, nY, nA))';
%adding up accross productivity shocks to get the distribution accross asset states
lambdasa = sum(lambdasmat,2);
lambdaua = sum(lambdaumat,2);
%sum(sum(lambdasmat,2)) + sum(sum(lambdaumat,2))

%aggregate effective labors
ls = sum(sum(kron(Ys, ones(nA,1)).*lambdasmat.*lspol));
lu = sum(sum(kron(Yu, ones(nA,1)).*lambdaumat.*lupol));

%average labor
%l1 = sum(sum(lambdasmat.*lspol))/pis;
%l2 = sum(sum(lambdaumat.*lupol))/piu;
lagg = sum(sum(lambdasmat.*lspol))+sum(sum(lambdaumat.*lupol));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%the new guesses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%total asset supply
Atemp = lambdasa'*A + lambdaua'*A - B;

%given ls and lu, find ke and ks that make sure the return is right
options = optimset('display', 'off', 'tolfun', 10^(-20), 'tolX', 10^(-20), 'maxfunevals', 1000000, 'maxiter', 1000000);
ktemp = fsolve(@ycompprimeopen, [.5, .5] * Atemp, options);
ks = ktemp(1);
ke = ktemp(2);
Knew = ks+ke;
%defining net foerign asset position
NFAP = Atemp - Knew;

%finding the individual ke and ks s.t. the nac holds
%options = optimset('display', 'off', 'tolfun', 10^(-20), 'tolX', 10^(-20), 'maxfunevals', 1000000, 'maxiter', 1000000);
%ke = fsolve(@nacdiff, .5*k0, options)
%ks = k0 - ke
%bla

x = [ks, ke, lu, ls];
gdp = ycomp(x);
KtoY = Knew/gdp;
NFAPtoY = NFAP/gdp;


%cagg = sum(sum(lambdasmat.*cspol)) + sum(sum(lambdaumat.*cupol));
%check2 = gdp-cagg - de*ke - ds*ks - G 

%constructing the new prices and transfers
MP = ycompprime2(x);
rsnew = MP(1);
renew = MP(2);
%Rnew = 1+(rsnew-ds)*(1-tauks)
wnew = [MP(3), MP(4)];
gdpnew = ycomp(x);
Bnew = by*gdpnew;

%gdp - gdpnew;

%mean pretax labor income for the tax function
pres = kron(Ys*wnew(2), ones(nA,1)).*lspol;
preu = kron(Yu*wnew(1), ones(nA,1)).*lupol;
meanynew = sum(sum(lambdasmat.*pres)) + sum(sum(lambdaumat.*preu));

%aftertax income given new wages and new mean income
Ysmat = lambdal*(kron(Ys*wnew(2), ones(nA,1)).*lspol/meanynew).^(1-taul)*meanynew;
Yumat = lambdal*(kron(Yu*wnew(1), ones(nA,1)).*lupol/meanynew).^(1-taul)*meanynew;

%aggregate consumption in new prices
cspol = (R*kron(A,ones(1,nY))-asprime + Ysmat)/(1+tauc);
cupol = (R*kron(A,ones(1,nY))-auprime + Yumat)/(1+tauc);
%c1 = sum(sum(lambdasmat.*cspol))/pis;
%c2 = sum(sum(lambdaumat.*cupol))/piu;
cagg = sum(sum(lambdasmat.*cspol)) + sum(sum(lambdaumat.*cupol));

%labor income taxes
taxs = pres - Ysmat;
taxu = preu - Yumat;
ltagg = sum(sum(lambdasmat.*taxs)) + sum(sum(lambdaumat.*taxu));

%gvt budget to pin down lambdal
budget = tauc*cagg + ltagg + tauks*(rsnew-ds)*ks + tauke*(renew-de)*ke - gy*gdpnew - (1-tauks)*(rsnew-ds)*Bnew;

%targets in the calibration procedure
sp = wnew(2)/wnew(1); %works if mean Y is 1 in each occupation
lshare = (lu*wnew(1) + ls*wnew(2))/gdp;
rks = lambdasa'*A/pis/(lambdaua'*A/piu);

%aiyagarical3b = [w - wnew, budget, B - Bnew, sp - 1.8, KtoY - 2.9, lshare - 2/3, rks - 2.68, meany - meanynew]
aiyagarical3b = [KtoY - 2, w - wnew, budget, B - Bnew, sp - 1.8, NFAPtoY+.2, lshare - 2/3, rks - 2.68, meany - meanynew];